"""
SecretColors 2019
Author: Rohit Suratekar

Utility functions used in this module.
These can also be used as standalone functions.
"""
import warnings


def _warn(message: str, show_warning: bool = True) -> None:
    """
    Simple function to generate warning
    :param message: Message you want to send
    :param show_warning: If False, warnings will be suppressed
    """
    if show_warning:
        m = message + "\nTo suppress warning use 'show_warning=False' in " \
                      "constructor of the palette"
        ##
        warnings.warn(m)


def __int_to_hex(num: int):
    """
    Converts integer to hex. Automatically rounds of the float
    :param num: Integer to be converted
    :return: Hex string
    """
    return '%02x' % int(num)


def rgb_to_hex(r, g, b):
    """
    Converts RGB tuple to hex. Ignores alpha channel
    All Red, Green, Blue values should be between 0 to 1

    >>> rgb_to_hex(0.251, 0.545, 0.988) # '#408afb'

    :param r: Red
    :param g: Green
    :param b: Blue
    :return: Hex color
    """
    return "#" + __int_to_hex(r * 255) + __int_to_hex(
        g * 255) + __int_to_hex(b * 255)


def hex_to_rgb(hex_color: str) -> tuple:
    """
    Converts hex color to RGB

    >>> hex_to_rgb("#408afb") # (0.251, 0.541, 0.984)

    :param hex_color: Color in Hex
    :return: (Red, Green, Blue) between 0 to 1
    """
    if len(hex_color.replace("#", "")) == 8:
        r, g, b, a = [hex_color.replace("#", "")[i:i + 2] for i in
                      range(0, 8, 2)]

        return round(int(r, 16) / 255, 3), round(int(g, 16) / 255, 3), round(
            int(b, 16) / 255, 3), int(a)
    elif len(hex_color.replace("#", "")) == 6:
        r, g, b = [hex_color.replace("#", "")[i:i + 2] for i in
                   range(0, 6, 2)]

        return round(int(r, 16) / 255, 3), round(int(g, 16) / 255, 3), round(
            int(b, 16) / 255, 3)

    elif len(hex_color.replace("#", "")) == 3:
        r, g, b = [x for x in hex_color.replace("#", "")]
        return round(int(r + r, 16) / 255, 3), round(int(g + g, 16) / 255,
                                                     3), round(
            int(b + b, 16) / 255, 3)

    else:
        raise Exception("Invalid Hex Code")


def hex_to_rgb255(hex_color: str) -> tuple:
    """
    Converts hex color to RGB with 0 to 255 range instead 0 to 1

    >>> hex_to_rgb255("#fb4b53") # (251, 75, 83)

    :param hex_color: Color in Hex
    :return: (Red, Green, Blue) between 0 to 255
    """
    if len(hex_color.replace("#", "")) == 8:
        r, g, b, a = [hex_color.replace("#", "")[i:i + 2] for i in
                      range(0, 8, 2)]

        return round(int(r, 16), 3), round(int(g, 16), 3), round(
            int(b, 16), 3), int(a)
    elif len(hex_color.replace("#", "")) == 6:
        r, g, b = [hex_color.replace("#", "")[i:i + 2] for i in
                   range(0, 6, 2)]

        return round(int(r, 16), 3), round(int(g, 16), 3), round(
            int(b, 16), 3)

    elif len(hex_color.replace("#", "")) == 3:
        r, g, b = [x for x in hex_color.replace("#", "")]
        return round(int(r + r, 16), 3), round(int(g + g, 16), 3), round(
            int(b + b, 16), 3)

    else:
        raise Exception("Invalid Hex Code")


def color_in_between(c1, c2, no_of_colors=1) -> list:
    """
    Creates color between two colors

    >>> color_in_between("#fb4b53", "#408bfc") # ['#9d6aa7']
    >>> color_in_between("#fb4b53", "#408bfc", 3) # ['#cc5a7d', '#9d6aa7', '#6e7ad1']


    :param c1: Hex of first color
    :param c2: Hex of second color
    :param no_of_colors: How many colors in between? [Default :1]
    :return: List of Colors between provided colors in RGB space
    """
    steps = no_of_colors + 1
    all_colors = []
    r1, g1, b1 = hex_to_rgb(c1)
    r2, g2, b2 = hex_to_rgb(c2)
    rdelta, gdelta, bdelta = (r2 - r1) / steps, (g2 - g1) / steps, (
            b2 - b1) / steps
    for step in range(steps - 1):
        r1 += rdelta
        g1 += gdelta
        b1 += bdelta
        all_colors.append(rgb_to_hex(r1, g1, b1))
    return all_colors


def get_complementary(hex_color: str):
    """
    Returns complementary color

    >>> get_complementary("#fb4b53") # '#4afaf3'

    :param hex_color: Hex color
    :return: Complementary Hex color
    """

    r, g, b = hex_to_rgb(hex_color)
    k = max([r, g, b]) + min([r, g, b])
    t = tuple(k - u for u in (r, g, b))
    return rgb_to_hex(t[0], t[1], t[2])


def text_color(hex_color: str):
    """
    Provides black or white color which can be used for text on given hex
    color background

    Taken from:
    https://stackoverflow.com/questions/3942878/how-to-decide-font-color-in-white-or-black-depending-on-background-color

    >>> text_color("#ffffff") # '#000000'

    :param hex_color: background color
    :return: proper text color
    """
    r, g, b = hex_to_rgb(hex_color)
    score = (r * 0.299 + g * 0.587 + b * 0.114)
    return "#000000" if score > 0.729 else "#ffffff"


def rgb_to_hsl(r, g, b) -> tuple:
    """
    Converts RGB tuple into HSL tuple
    Calculations are taken from  http://www.easyrgb.com/en/math.php

    :param r: Red (between 0 to 1)
    :param g: Green (between 0 to 1)
    :param b: Blue (between 0 to 1)
    :return: (hue, saturation, lightness) All between 0 to 1
    """
    min_rgb = min(r, g, b)
    max_rgb = max(r, g, b)
    delta_rgb = max_rgb - min_rgb
    l = (max_rgb + min_rgb) / 2
    if delta_rgb == 0:
        h = 0
        s = 0
    else:
        if l < 0.5:
            s = delta_rgb / (max_rgb + min_rgb)
        else:
            s = delta_rgb / (2 - max_rgb - min_rgb)

        delta_r = (((max_rgb - r) / 6) + (delta_rgb / 2)) / delta_rgb
        delta_g = (((max_rgb - g) / 6) + (delta_rgb / 2)) / delta_rgb
        delta_b = (((max_rgb - b) / 6) + (delta_rgb / 2)) / delta_rgb

        if r == max_rgb:
            h = delta_b - delta_g
        elif g == max_rgb:
            h = (1 / 3) + delta_r - delta_b
        else:
            h = (2 / 3) + delta_g - delta_r

        if h < 0:
            h += 1
        if h > 1:
            h -= 1

    return round(h, 3), round(s, 3), round(l, 3)


def __hue_to_rgb(v1, v2, vh):
    """
    Calculation from http://www.easyrgb.com/en/math.php
    """
    if vh < 0:
        vh += 1
    if vh > 1:
        vh -= 1
    if 6 * vh < 1:
        return v1 + ((v2 - v1) * 6 * vh)
    elif 2 * vh < 1:
        return v2
    elif 3 * vh < 2:
        return v1 + ((v2 - v1) * ((2 / 3) - vh) * 6)
    else:
        return v1


def hsl_to_rgb(h, s, l) -> tuple:
    """
    Converts HSL values to RGB tuple.
    Calculations are taken from http://www.easyrgb.com/en/math.php

    :param h: Hue (between 0 to 1)
    :param s: Saturation (between 0 to 1)
    :param l: Lightness (between 0 to 1)
    :return: (Red, Green, Blue) between 0 to 1
    """
    if s == 0:
        r = l
        g = l
        b = l
    else:
        if l < 0.5:
            var2 = l * (1 + s)
        else:
            var2 = (l + s) - (s * l)

        var1 = (2 * l) - var2

        r = __hue_to_rgb(var1, var2, h + (1 / 3))
        g = __hue_to_rgb(var1, var2, h)
        b = __hue_to_rgb(var1, var2, h - (1 / 3))

    return round(r, 3), round(g, 3), round(b, 3)


def hex_to_hsl(hex_name: str) -> tuple:
    """
    Converts Hex Color to HSL tuple

    :param hex_name: Color code in Hex
    :return: (Hue, Saturation, Lightness) between 0 to 1
    """
    r, g, b = hex_to_rgb(hex_name)
    return rgb_to_hsl(r, g, b)


def hsl_to_hex(h, s, l):
    """
    Converts HSL to Hex

    :param h: Hue (between 0 to 1)
    :param s: Saturation (between 0 to 1)
    :param l: Lightness (between 0 to 1)
    :return: Hex color
    """
    r, g, b = hsl_to_rgb(h, s, l)
    return rgb_to_hex(r, g, b)


def hex_to_ahex(hex_name: str, alpha: float):
    """
    Adds Transparency unit to hex code

    :param hex_name: Color in hex format
    :param alpha: Transparency between 0 to 1
    :return: Hex code with transparency value
    """
    if alpha > 1 or alpha < 0:
        raise Exception("Alpha value should be between 0 and 1")
    if len(hex_name) == 7:
        return "#{}{}".format(__int_to_hex(int(alpha * 255)), hex_name[1:7])
    else:
        raise Exception("Invalid Hex")


def hex_to_hex_a(hex_name: str, alpha: float):
    """
    Adds Transparency unit to hex code

    :param hex_name: Color in hex format
    :param alpha: Transparency between 0 to 1
    :return: Hex code with transparency value
    """
    if alpha > 1 or alpha < 0:
        raise Exception("Alpha value should be between 0 and 1")
    if len(hex_name) == 7:
        return "#{}{}".format(hex_name[1:7], __int_to_hex(int(alpha * 255)))
    else:
        raise Exception("Invalid Hex")


def hex_to_rgba(hex_name: str, alpha: float):
    """
    Hex to RGBA

    :param hex_name: Color in hex format
    :param alpha: Transparency between 0 to 1
    :return: (Red, Green, Blue, Alpha) all between 0 to 1
    """
    if alpha > 1 or alpha < 0:
        raise Exception("Alpha value should be between 0 and 1")
    r, b, g = hex_to_rgb(hex_name)
    return r, g, b, alpha


def hex_to_hsla(hex_name: str, alpha: float):
    """
    Hex to HSLA

    :param hex_name: Color in hex format
    :param alpha: Transparency between 0 to 1
    :return: (Hue, Saturation, Lightness, Alpha) all between 0 to 1
    """
    if alpha > 1 or alpha < 0:
        raise Exception("Alpha value should be between 0 and 1")
    h, s, l = hex_to_hsl(hex_name)
    return h, s, l, alpha



"""
Author: Rohit Suratekar

These are functions used in generating plots on https://weirddata.github.io/
Blog Post: Matplotlib vs SecretColors
"""

import matplotlib
import matplotlib.pylab as plt
import numpy as np

from SecretColors import Palette, ColorMap


# import seaborn as sns


def bar_plots():
    p = Palette()
    data = np.random.random(6)
    mat_colors = ["r", "b", "g", "y", "m", "c"]
    ibm_colors = [p.red(), p.blue(), p.green(), p.yellow(), p.magenta(),
                  p.cyan()]

    plt.subplot(121)

    for i, v in enumerate(data):
        plt.bar(i, v, color=mat_colors[i])
        plt.xticks([])
        plt.yticks([])
        plt.title("Matplotlib")

    plt.subplot(122)
    for i, v in enumerate(data):
        plt.bar(i, v, color=ibm_colors[i])
        plt.xticks([])
        plt.yticks([])
        plt.title("SecretColors")

    plt.show()


def bar_plot_color_cycle():
    p = Palette()
    data = np.random.random(10)

    plt.subplot(121)

    for i, v in enumerate(data):
        plt.bar(i, v)
        plt.xticks([])
        plt.yticks([])
        plt.title("Matplotlib")

    colors = p.get_color_list
    plt.subplot(122)
    for i, v in enumerate(data):
        plt.bar(i, v, color=colors[i])
        plt.xticks([])
        plt.yticks([])
        plt.title("SecretColors")

    plt.show()


def all_default_colors():
    p = Palette()
    colors = p.get_color_list

    for i, v in enumerate(colors):
        plt.bar(i, np.random.random(), color=v)
        plt.xticks([])
        plt.yticks([])
        plt.title("IBM Full Color Cycle")

    plt.show()


def test_gradient():
    ibm = Palette()
    x_data = range(0, 100, 10)
    ind = range(len(x_data))
    plt.figure(figsize=(9, 6))

    cmap = matplotlib.cm.get_cmap('Greys')
    plt.subplot(121)
    colors = [cmap(x / max(x_data)) for x in x_data]
    plt.bar(ind, x_data, color=colors)
    plt.xticks([])
    plt.yticks([])
    plt.title("Default")

    plt.subplot(122)
    new_colors = [ibm.gray(shade=int(x)) for x in x_data]
    plt.bar(ind, x_data, color=new_colors)
    plt.xticks([])
    plt.yticks([])
    plt.title("SecretColors")

    plt.show()


def test_lines():
    ibm = Palette()
    all_data = []
    for d in range(4):
        data = np.random.random(30) + d * 0.8
        all_data.append(data)
    time = range(len(all_data[0]))
    plt.figure(figsize=(9, 6))

    plt.subplot(121)
    colors = ["r", "g", "b", "y"]
    i = 0
    for data in all_data:
        plt.plot(time, data, linewidth=4, color=colors[i])
        i += 1
    plt.title("Default")

    plt.subplot(122)
    new_colors = [ibm.red(), ibm.green(), ibm.blue(), ibm.yellow()]
    i = 0
    for data in all_data:
        plt.plot(time, data, linewidth=4, color=new_colors[i])
        i += 1
    plt.title("SecretColors")

    plt.show()


def histograms():
    ibm = Palette()
    mu, sigma = 100, 15
    mu2, sigma2 = 150, 20
    x = mu + sigma * np.random.randn(10000)
    y = mu2 + sigma2 * np.random.randn(10000)

    plt.subplot(131)
    plt.hist(x, 30, color="m")
    plt.hist(y, 30, color="c")
    plt.yticks([])
    plt.xticks([])
    plt.title("Matplotlib")

    plt.subplot(132)
    plt.hist(x, 30, color=ibm.magenta())
    plt.hist(y, 30, color=ibm.cyan())
    plt.yticks([])
    plt.xticks([])
    plt.title("SecretColors Defaults")

    plt.subplot(133)
    plt.hist(x, 30, color=ibm.magenta(shade=60))
    plt.hist(y, 30, color=ibm.cyan(shade=40))
    plt.yticks([])
    plt.xticks([])
    plt.title("SecretColors Modified")

    plt.show()


def histograms_shades():
    ibm = Palette()
    mu, sigma = 100, 15
    mu2, sigma2 = 150, 20
    x = mu + sigma * np.random.randn(10000)
    y = mu2 + sigma2 * np.random.randn(10000)

    plt.subplot(131)
    plt.hist(x, 30, color=ibm.magenta(shade=30))
    plt.hist(y, 30, color=ibm.cyan())
    plt.yticks([])
    plt.xticks([])
    plt.title("shade=30")

    plt.subplot(132)
    plt.hist(x, 30, color=ibm.magenta(shade=60))
    plt.hist(y, 30, color=ibm.cyan())
    plt.yticks([])
    plt.xticks([])
    plt.title("shade=60")

    plt.subplot(133)
    plt.hist(x, 30, color=ibm.magenta(shade=80))
    plt.hist(y, 30, color=ibm.cyan())
    plt.yticks([])
    plt.xticks([])
    plt.title("shade=80")

    plt.show()


def heatmap_standard():
    ibm = Palette()
    cmap = ColorMap(matplotlib, ibm)
    a = np.random.random((16, 16))

    plt.subplot(121)
    plt.imshow(a, cmap='Greens', interpolation='nearest')
    plt.colorbar()
    plt.title("Matplotlib")

    plt.subplot(122)
    plt.imshow(a, cmap=cmap.greens(), interpolation='nearest')
    plt.colorbar()
    plt.title("SecretColors")

    plt.show()


def heatmap_custom():
    ibm = Palette(show_warning=False)
    cmap = ColorMap(matplotlib, ibm)
    a = np.random.random((16, 16))

    plt.subplot(121)
    colors = [ibm.green(shade=0), ibm.green(shade=50)]
    plt.imshow(a, cmap=cmap.from_list(colors), interpolation='nearest')
    plt.colorbar()
    plt.title("Lighter Shades")

    plt.subplot(122)
    colors = [ibm.green(shade=50), ibm.green(shade=100)]
    plt.imshow(a, cmap=cmap.from_list(colors), interpolation='nearest')
    plt.colorbar()
    plt.title("Darker Shades")

    plt.show()


def shift_colormap():
    ibm = Palette(show_warning=False)
    cmap = ColorMap(matplotlib, ibm)
    a = np.random.random((16, 16))

    plt.subplot(131)
    colors = [ibm.green(shade=0), ibm.green(shade=30), ibm.green(shade=100)]
    plt.imshow(a, cmap=cmap.from_list(colors), interpolation='nearest')
    plt.colorbar()
    plt.title("Shades : 0-30-100")

    plt.subplot(132)
    colors = [ibm.green(shade=0), ibm.green(shade=50), ibm.green(shade=100)]
    plt.imshow(a, cmap=cmap.from_list(colors), interpolation='nearest')
    plt.colorbar()
    plt.title("Shades : 0-50-100")

    plt.subplot(133)
    colors = [ibm.green(shade=0), ibm.green(shade=70), ibm.green(shade=100)]
    plt.imshow(a, cmap=cmap.from_list(colors), interpolation='nearest')
    plt.colorbar()
    plt.title("Shades : 0-70-100")

    plt.show()


def heatmap_custom2():
    ibm = Palette(show_warning=False)
    cmap = ColorMap(matplotlib, ibm)
    a = np.random.random((16, 16))

    plt.subplot(121)
    colors = [ibm.purple(shade=80), ibm.purple(shade=40), ibm.yellow(
        shade=30), ibm.yellow(shade=50)]
    plt.imshow(a, cmap=cmap.from_list(colors), interpolation='nearest')
    plt.colorbar()
    plt.title("Yellow-Purple")

    plt.subplot(122)
    colors = [ibm.lime(shade=30), ibm.brown(), ibm.black()]
    plt.imshow(a, cmap=cmap.from_list(colors), interpolation='nearest')
    plt.colorbar()
    plt.title("Black-Brown-Lime")

    plt.show()


def heatmap_qualitative():
    ibm = Palette(show_warning=False)
    cmap = ColorMap(matplotlib, ibm)
    a = np.random.random((16, 16))

    plt.subplot(121)
    colors = [ibm.green(30), ibm.green(80)]
    plt.imshow(a, cmap=cmap.from_list(colors, is_qualitative=True),
               interpolation='nearest')
    plt.colorbar()
    plt.title("Two colors")

    plt.subplot(122)
    colors = ibm.green(no_of_colors=5)
    plt.imshow(a, cmap=cmap.from_list(colors, is_qualitative=True),
               interpolation='nearest')
    plt.colorbar()
    plt.title("Five colors")

    plt.show()


# def testing_seaborn():
#     p = Palette()
#     ind = [str(x) for x in range(10)]
#     data = np.random.random(10)
#     colors = {str(p.get_color_list.index(v)): v for v in p.get_color_list[:10]}
#     plt.subplot(121)
#     sns.barplot(x=ind, y=data)
#     plt.title("Seaborn Default")
#     plt.xticks([])
#     plt.yticks([])
#     plt.subplot(122)
#     sns.barplot(x=ind, y=data, palette=colors)
#     plt.title("SecretColors Default")
#     plt.xticks([])
#     plt.yticks([])
#     plt.show()


p = Palette()
print(p.red())



"""
SecretColors 2019
Author: Rohit Suratekar

Color and its related classes
"""

import random

import math

from SecretColors.utils import color_in_between, _warn


class Shade:
    """
    Class to accommodate shades
    """

    def __init__(self, color: str, value: float, max_shade: float):
        """
        :param color: Name of the color
        :param value: Value of the shade
        :param max_shade: Maximum shade possible in current palette
        """
        self.hex = color
        self.__value = value
        self.__max_shade = max_shade

    @property
    def value(self):
        """
        :return: Normalized shade value
        """
        return round(self.__value * 100 / self.__max_shade)


class Color:
    """
    Base Color Class
    """

    def __init__(self, data: dict, shades: list, core: int,
                 show_warning: bool = True):
        """
        :param data: Dictionary values from __colors.py
        :param shades: List of standard shades
        :param core: Shade of core color (Default Shade)
        :param show_warning: If True, warnings will be shown
        """

        self._raw = data
        self.name = data['n'].lower()  # Name
        self.type = data['t'].lower()  # Type
        self.__raw_shade_colors = data['c']
        self.__default_shade_values = shades
        self.__raw_core = core
        self.__max_shade = math.pow(10, math.ceil(math.log10(max(shades))))
        self.show_warnings = show_warning

        if len(shades) > len(self.__raw_shade_colors):
            for i in range(len(shades) - len(self.__raw_shade_colors)):
                self.__raw_shade_colors.append(self.__raw_shade_colors[-1])

        self.__default_shades_dict = {}
        for i, s in enumerate(self.__raw_shade_colors):
            x = Shade(s, self.__default_shade_values[i], self.__max_shade)
            self.__default_shades_dict[x.value] = x

    @property
    def shade_slabs(self) -> list:
        """
        :return: Standard normalized shade slabs
        """
        return [round(x * 100 / self.__max_shade)
                for x in self.__default_shade_values]

    @property
    def core_shade_value(self) -> int:
        """
        :return: Normalized value of core shade
        """
        return round(self.__raw_core * 100 / self.__max_shade)

    @property
    def default_shades_list(self) -> list:
        """
        :return: List of default shades
        """
        return [x for x in self.__default_shades_dict.values()]

    @property
    def hex(self):
        """
        :return: Hex value of default shade
        """
        try:
            return self.__default_shades_dict[self.core_shade_value].hex
        except KeyError:
            for x in self.__default_shades_dict.values():
                return x.hex

    def __str__(self):
        """
        :return: Hex value of core shade
        """
        return self.hex

    def shade(self, value: float = None) -> str:
        """
        :param value: Shade percentage (min and max is defined in the palette)
        :return: Hex code for given shade
        """

        if value is None:
            return self.__default_shades_dict[self.core_shade_value].hex
        else:
            if value > 100:
                _warn("Maximum allowed shade is 100. Provided value is {}. "
                      "Returning black".format(value), self.show_warnings)
                # If value is more than or equal to 100, return black
                return "#000000"
            elif value < 0:
                _warn("Minimum allowed shade is 0. Provided value is {}. "
                      "Returning white".format(value), self.show_warnings)
                # If value is 0, return white
                return "#ffffff"
            elif int(value) in self.shade_slabs:
                # If value is available in standard shade slabs, return it
                return self.__default_shades_dict[int(value)].hex
            else:
                # If value is greater than max slab, return color between max
                # shade and black
                if value > max(self.shade_slabs):
                    _warn(
                        "Maximum shade of of this palette is {} and provided "
                        "shade value is {} , hence black color is used to "
                        "generate darker shade".format(
                            max(self.shade_slabs), value), self.show_warnings)
                    col3 = color_in_between(
                        self.__default_shades_dict[max(self.shade_slabs)].hex,
                        "#000000", 98)
                    col3.insert(0, self.__default_shades_dict[
                        max(self.shade_slabs)].hex)
                    col3.append("#000000")
                    ind3 = 100 - round(1000 - value * 10)
                    return col3[ind3 - 1]
                for i, s in enumerate(self.shade_slabs):
                    if value > s:
                        cols = color_in_between(
                            self.__default_shades_dict[s].hex,
                            self.__default_shades_dict[
                                self.shade_slabs[i - 1]].hex,
                            99)
                        cols.append(self.__default_shades_dict[
                                        self.shade_slabs[i - 1]].hex)
                        ind = round((value - s) * 10)
                        return cols[ind - 1]

            if value == 0:
                _warn(
                    "Minimum shade of this palette is {} and provided shade "
                    "value is {} , hence white is returned.".format(
                        min(self.shade_slabs), value), self.show_warnings)

                return "#ffffff"

            # If value is below minimum standard shade, use white as a first
            # color and then calculate the shade

            _warn(
                "Minimum shade of this palette is {} and provided shade "
                "value is {} , hence white color is used to generate lighter "
                "shade".format(
                    min(self.shade_slabs), value), self.show_warnings)

            col2 = color_in_between("#ffffff", self.__default_shades_dict[
                min(self.shade_slabs)].hex, 98)

            col2.insert(0, "#ffffff")
            col2.append(self.__default_shades_dict[min(self.shade_slabs)].hex)

            ind2 = round(100 - (min(self.shade_slabs) - value) * 100 / min(
                self.shade_slabs))

            return col2[ind2]

    def random_between(self, starting_shade: float, ending_shade: float,
                       no_of_colors: int = 1):
        """
        Generates colors between two shades of the current color

        :param starting_shade: Minimum shade (0-100)
        :param ending_shade: Maximum Shade (0-100)
        :param no_of_colors: Number of colors
        :return: Color/List of colors (in Hex)
        """
        if starting_shade < 0 or ending_shade > 100:
            raise Exception("Shade value should be between 0 and 100")
        else:
            random_shades = []
            for i in range(no_of_colors):
                random_shades.append(
                    random.uniform(starting_shade, ending_shade))

            return [self.shade(x) for x in random_shades]

    def gradient(self, starting_shade: float, ending_shade: float,
                 no_of_colors: int = 2, include_first: bool = False,
                 include_last: bool = False,
                 include_both: bool = False) -> list:

        """
        Generates the color gradient

        :param starting_shade: Minimum shade
        :param ending_shade: Maximum Shade
        :param no_of_colors: Number of colors
        :param include_first: If True, starting shade will be included
        :param include_last: If True, end shade will be included
        :param include_both: If True, both starting and ending shade will be
        included in the final list of colors
        :return: List of Colors
        """
        if no_of_colors < 2:
            raise Exception("No of colors should be minimum 2")

        if no_of_colors < 3:
            if include_both or (include_first and include_last):
                return [self.shade(starting_shade), self.shade(ending_shade)]
            elif include_first:
                return [self.shade(starting_shade),
                        self.shade(
                            random.uniform(starting_shade, ending_shade))]
            elif include_last:
                return [self.shade(
                    random.uniform(starting_shade, ending_shade)),
                    self.shade(ending_shade)]

        grad_shades = []
        if include_both or (include_first and include_last):
            step_count = -2
            grad_shades.append(starting_shade)
        elif include_first:
            step_count = -1
            grad_shades.append(starting_shade)
        elif include_last:
            step_count = -1
        else:
            step_count = 0

        step = (ending_shade - starting_shade) / (no_of_colors + step_count + 1)

        for i in range(no_of_colors + step_count):
            if len(grad_shades) > 0:
                grad_shades.append(grad_shades[-1] + step)
            else:
                grad_shades.append(starting_shade + step)

        if include_last or include_both:
            grad_shades.append(ending_shade)

        return [self.shade(x) for x in grad_shades]
